use std::error::Error;

use crate::number::Number;

#[derive(Debug, PartialEq, Eq)]
pub struct Fraction {
    pub numerator: Number,
    pub denominator: Number,
}

pub struct Rational {}

impl Fraction {
    pub fn new(numerator: Number, denominator: Number) -> Result<Self, Box<dyn Error>> {
        if denominator == 0.into() {
            return Err(Box::from("Division by 0"));
        }
        let mut f = Fraction {
            numerator,
            denominator,
        };
        f.reduce();
        Ok(f)
    }

    pub fn reduce(&mut self) {
        let gcd = Number::gcd(self.numerator.clone(), self.denominator.clone());
        if gcd == self.denominator.clone() {
            return;
        }
        self.numerator = self.numerator.clone() / gcd.clone();
        self.denominator = self.denominator.clone() / gcd.clone();
    }
}

#[cfg(test)]
mod test {
    use super::Fraction;

    #[test]
    fn test_reduce() {
        let f = Fraction::new(120.into(), 90.into()).unwrap();
        assert_eq!(
            f,
            Fraction {
                numerator: 4.into(),
                denominator: 3.into()
            }
        );
    }
}
